package com.ekexiu.project.storage.diacrisis;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.TimeUnit;

import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.LoginUser;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.PathVar;
import org.jfw.apt.web.annotation.param.Upload;
import org.jfw.util.ArrayUtil;
import org.jfw.util.ConstData;
import org.jfw.util.ListUtil;
import org.jfw.util.PageQueryResult;
import org.jfw.util.StringUtil;
import org.jfw.util.context.JfwAppContext;
import org.jfw.util.io.IoUtil;
import org.jfw.util.jdbc.JdbcUtil;
import org.jfw.util.json.JsonService;
import org.jfw.util.log.LogFactory;
import org.jfw.util.log.Logger;
import org.jfw.util.reflect.TypeReference;
import org.jfw.util.web.fileupload.Item;
import org.jfw.util.web.fileupload.UploadItemIterator;

import com.ekexiu.project.hdfs.HdfsService;
import com.ekexiu.project.storage.system.SessionUser;
import com.google.gson.Gson;

@Path("/task")
public class AiTaskService {

	private static Logger LOG = LogFactory.getLog(AiTaskService.class);

	private long delayTime = 2;

	@Autowrie
	private HdfsService hdfsService;

	@Autowrie
	private AiTaskDao aiTaskDao;
	private File srcPath = new File("/ekexiu/storage_asd/src");
	private File dstPath = new File("/ekexiu/storage_asd/dst");
	private File uploadPath = new File("/ekexiu/storage_asd/upload");

	public HdfsService getHdfsService() {
		return hdfsService;
	}

	public void setHdfsService(HdfsService hdfsService) {
		this.hdfsService = hdfsService;
	}

	public File getUploadPath() {
		return uploadPath;
	}

	public void setUploadPath(File uploadPath) {
		this.uploadPath = uploadPath;
	}

	public File getSrcPath() {
		return srcPath;
	}

	public void setSrcPath(File srcPath) {
		this.srcPath = srcPath;
	}

	public File getDstPath() {
		return dstPath;
	}

	public void setDstPath(File dstPath) {
		this.dstPath = dstPath;
	}

	public AiTaskDao getAiTaskDao() {
		return aiTaskDao;
	}

	public void setAiTaskDao(AiTaskDao aiTaskDao) {
		this.aiTaskDao = aiTaskDao;
	}

	public void uploadToHdfs(final String dst, final File src) {
		JfwAppContext.getScheduledExecutorService().submit(new Runnable() {
			@Override
			public void run() {
				try {
					hdfsService.upload(dst, src);
				} catch (Throwable tr) {
				}
			}
		});
	}

	public boolean copyFile(File src, File dst) {
		try {
			if (src.exists() && (!dst.exists())) {
				InputStream in = null;
				OutputStream out = null;
				try {
					in = new FileInputStream(src);
					out = new FileOutputStream(dst);
					byte[] buf = new byte[4096];
					IoUtil.copy(in, out, buf);
					out.flush();
				} finally {
					if (in != null) {
						IoUtil.close(in);
					}
					if (out != null) {
						IoUtil.close(out);
					}
				}
				return true;
			}
		} catch (Exception e) {
		}
		return false;
	}

	public void updateTaskState(String id, int state, String stateCode, final Runnable runnable) {
		Connection con;
		try {
			con = JfwAppContext.getDataSource().getConnection();
			try {
				if (stateCode != null) {
					aiTaskDao.update(con, id, state, stateCode);
				} else {
					aiTaskDao.update(con, id, state);
				}
				con.commit();
				if (runnable != null) {
					JfwAppContext.getScheduledExecutorService().schedule(runnable, this.delayTime, TimeUnit.MILLISECONDS);
				}
			} catch (Exception e) {
				JdbcUtil.rollback(con);
			} finally {
				con.close();
			}
		} catch (Exception e1) {
		}
	}

	@Path("/upload")
	@Post
	public List<UploadFile> upload(@Upload UploadItemIterator it) throws Exception {
		List<UploadFile> ret = new LinkedList<UploadFile>();
		try {
			while (it.hasNext()) {
				Item item = it.next();
				if (!item.isFormField()) {
					String name = normalizeFileName(item.getName());
					int index = name.lastIndexOf('.');
					String ext = index >= 0 ? name.substring(index + 1) : "";
					ext = ext.trim();
					long fileLen = 0;
					int len = 0;
					UploadFile uf = this.buildTargetFile(ext);
					uf.setName(name);
					File file = uf.getFn();
					ret.add(uf);
					byte[] buf = new byte[8092];
					OutputStream of = new FileOutputStream(file);
					try {
						InputStream in = item.getInputStream();
						try {
							while ((len = in.read(buf)) >= 0) {
								if (len > 0) {
									of.write(buf, 0, len);
									fileLen += len;
								}
							}
							uf.setSize(fileLen);
						} finally {
							in.close();
						}
					} finally {
						of.close();
					}
				}
			}
			return ret;
		} catch (Exception e) {
			this.remove(ret);
			throw e;
		} finally {
			if (it != null)
				it.clean();
		}
	}

	@Path("/add")
	@Get
	@Post
	public String insert(@JdbcConn(true) Connection con, @LoginUser SessionUser user, final String dataNum, final int taskType) throws SQLException {
		final String id = StringUtil.buildUUID();
		AiTask task = new AiTask();

		task.setDataNum(dataNum);
		task.setTaskType(taskType);
		task.setState(100);
		task.setStateCode(null);
		task.setId(id);
		task.setUid(user.getId());
		aiTaskDao.insert(con, task);

		JfwAppContext.getScheduledExecutorService().schedule(new Runnable() {
			@Override
			public void run() {
				updateTaskState(id, 200, null, new AiTransferTask(id, taskType, AiTaskService.this));
			}
		}, this.delayTime, TimeUnit.MILLISECONDS);
		return id;
	}

	@Path("/new")
	@Get
	@Post
	public String insert(@JdbcConn(true) Connection con, @LoginUser SessionUser user, final int taskType, String[] file) throws IOException, SQLException {
		File path;
		String taskId;
		do {
			path = new File(this.dstPath, StringUtil.buildUUID());
		} while (path.exists());
		if (!path.mkdir()) {
			throw new RuntimeException("mkdir[" + path.getAbsolutePath() + "] error");
		}
		taskId = path.getName();
		path = new File(path, "data");
		if (!path.mkdir()) {
			throw new RuntimeException("mkdir[" + path.getAbsolutePath() + "] error");
		}

		for (String f : file) {
			int idx = f.indexOf('.');
			if (idx != 32) {
				throw new IllegalArgumentException("invalid param file");
			}
			File taskSrc = new File(this.uploadPath, f.substring(0, 32));
			File taskDst = new File(path, f.substring(33));
			InputStream in = new FileInputStream(taskSrc);
			try {
				OutputStream os = new FileOutputStream(taskDst);
				try {
					IoUtil.copy(in, os, false, false);
				} finally {
					IoUtil.close(os);
				}
			} finally {
				IoUtil.close(in);
			}
		}

		final String id = taskId;
		AiTask task = new AiTask();
		task.setDataNum(id.substring(24));
		task.setTaskType(taskType);
		task.setState(100);
		task.setStateCode(null);
		task.setId(id);
		task.setUid(user.getId());
		aiTaskDao.insert(con, task);

		JfwAppContext.getScheduledExecutorService().schedule(new Runnable() {
			@Override
			public void run() {
				updateTaskState(id, 200, null, new AiTransferTask(id, taskType, AiTaskService.this));
			}
		}, this.delayTime, TimeUnit.MILLISECONDS);
		return id;
	}

	private static final String EMPTY_STRING_ARRAY[] = new String[0];

	@Get
	@Path("/pq")
	public PageQueryResult<AiTask> pageQuery(@JdbcConn Connection con, @LoginUser SessionUser user, @Nullable String dataNum, @Nullable Integer taskType,
			int pageSize, int pageNo) throws SQLException {
		return aiTaskDao.pageQuery(con, user.getId(), dataNum, taskType, pageSize, pageNo);
	}

	@Get
	@Path("/result/{taskid}")
	public String[] getResult(@PathVar String taskid) {
		File path = new File(dstPath, taskid);
		if (path.exists() && path.isDirectory()) {
			path = new File(path, "RESULT");
			if (path.exists() && path.isDirectory()) {
				return path.list();
			}
		}
		return EMPTY_STRING_ARRAY;
	}

	@Get
	@Path("/qo")
	public AiTask query(@JdbcConn Connection con, String id) throws SQLException {
		return aiTaskDao.query(con, id);
	}

	private static final String[] EMTPY_STRING_ARRAY = new String[0]; 
	private static final Gson gson = new Gson();
	@Get
	@Path("/searchJpg")
	public List<String> serch(String id) throws IOException {
		List<String> sret = new LinkedList<String>();
		URL u = new URL("http://127.0.0.1:5005/test?pi=" + "/ekexiu/storage_asd/dst/" + id + "/data_trans");
		HttpURLConnection httpURLConnection = (HttpURLConnection) u.openConnection();
		if (httpURLConnection.getResponseCode() == 200) {
			InputStream in = httpURLConnection.getInputStream();
			try {
				ByteArrayOutputStream out = new ByteArrayOutputStream();
				byte[] buf = new byte[128];
				int len = 0;
				while ((len = in.read(buf)) != -1) {
					if (len > 0) {
						out.write(buf, 0, len);
					}
				}
				buf = out.toByteArray();
				if (buf.length > 0) {
					String ret = new String(buf, ConstData.UTF8);
					ret = ret.trim();
					if(ret.equals("0")){
						return Collections.<String>emptyList();
					}
					if(ret.startsWith("[") && ret.endsWith("]")){
						Object[] os=JsonService.fromJson(ret,Object[].class);
						for(Object o:os){
							if(o instanceof Collection<?>){
								Collection<?> ll =(Collection<?>)o;
								if(ll.size()==3){
									Iterator<?> it = ll.iterator();
									it.hasNext();
									it.next();
									it.hasNext();
									it.next();
									if(it.hasNext()){
										Object so = it.next();
										if(so instanceof String){
											sret.add((String)so);
										}
									}
								}
							}
						}
						return sret;
					}

				}

			} finally {
				IoUtil.close(in);
			}
		}
		throw new IOException("call 127.0.0.1 error");
	}

	
	@Get
	@Path("/searchTxt")
	public List<String> serchTxt(String txt) throws IOException {
		List<String> sret = new LinkedList<String>();
		URL u = new URL("http://127.0.0.1:5004/test?pi=" +URLEncoder.encode(txt,"UTF-8"));
		HttpURLConnection httpURLConnection = (HttpURLConnection) u.openConnection();
		if (httpURLConnection.getResponseCode() == 200) {
			InputStream in = httpURLConnection.getInputStream();
			try {
				ByteArrayOutputStream out = new ByteArrayOutputStream();
				byte[] buf = new byte[128];
				int len = 0;
				while ((len = in.read(buf)) != -1) {
					if (len > 0) {
						out.write(buf, 0, len);
					}
				}
				buf = out.toByteArray();
				if (buf.length > 0) {
					String ret = new String(buf, ConstData.UTF8);
					ret = ret.trim();
					if(ret.equals("0")){
						return Collections.<String>emptyList();
					}
					if(ret.startsWith("[") && ret.endsWith("]")){
						return JsonService.fromJson(ret,new TypeReference<List<String>>() {
						}.getType());
					}

				}

			} finally {
				IoUtil.close(in);
			}
		}
		throw new IOException("call 127.0.0.1 error");
	}
	private void remove(List<UploadFile> list) {
		for (UploadFile file : list) {
			if (file.fn != null)
				file.fn.delete();
		}
	}

	private UploadFile buildTargetFile(String ext) {
		File file;
		do {
			String fn = StringUtil.buildUUID();
			file = new File(this.uploadPath, StringUtil.buildUUID());
		} while (file.exists());
		UploadFile uf = new UploadFile();
		uf.setFn(file);
		uf.setUri(file.getName());
		return uf;
	}

	private String normalizeFileName(String fn) {
		int index = fn.indexOf('\\');
		if (index >= 0) {
			fn = fn.substring(fn.lastIndexOf('\\') + 1);
		}
		index = fn.indexOf('/');
		if (index >= 0) {
			fn = fn.substring(fn.lastIndexOf('/') + 1);
		}
		if (fn.length() == 0)
			throw new RuntimeException("invalid filename in Multipart/data request");
		return fn;
	}

	public static class UploadFile {
		private String name;
		private String uri;
		private long size;
		private transient File fn;

		public File getFn() {
			return fn;
		}

		public void setFn(File fn) {
			this.fn = fn;
		}

		public String getName() {
			return name;
		}

		public void setName(String name) {
			this.name = name;
		}

		public String getUri() {
			return uri;
		}

		public void setUri(String uri) {
			this.uri = uri;
		}

		public long getSize() {
			return size;
		}

		public void setSize(long size) {
			this.size = size;
		}
	}

	
	public static void main(String[] args){
		String s ="[\"aa\",[\"0\",1,\"0\"],[\"0\",0]]";
		
		Object[] os=JsonService.fromJson(s,Object[].class);
		for(Object o:os){
			System.out.println(o.getClass().getName());
			if(o instanceof Collection<?>){
				for(Object oo:(Collection<?>)o){
					System.out.println("========"+oo.getClass().getName());
				}
			}
		}
		
		
	}
}
