package com.ekexiu.project.storage.dicom;

import java.io.File;
import java.io.IOException;
import java.security.GeneralSecurityException;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;

import org.dcm4che3.data.Attributes;
import org.dcm4che3.data.Tag;
import org.dcm4che3.data.UID;
import org.dcm4che3.data.VR;
import org.dcm4che3.io.DicomOutputStream;
import org.dcm4che3.net.Association;
import org.dcm4che3.net.DimseRSPHandler;
import org.dcm4che3.net.IncompatibleConnectionException;
import org.dcm4che3.net.PDVInputStream;
import org.dcm4che3.net.Priority;
import org.dcm4che3.net.Status;
import org.dcm4che3.net.pdu.PresentationContext;
import org.dcm4che3.net.pdu.RoleSelection;
import org.dcm4che3.net.service.BasicCStoreSCP;
import org.dcm4che3.net.service.DicomServiceException;
import org.dcm4che3.net.service.DicomServiceRegistry;
import org.dcm4che3.util.SafeClose;

public class DicomGetImage extends DicomBase {
	private File storageDir;
	private String fn;
	private File dstFile;

	private BasicCStoreSCP storageSCP = new BasicCStoreSCP("*") {

		@Override
		protected void store(Association as, PresentationContext pc, Attributes rq, PDVInputStream data, Attributes rsp)
				throws IOException {
			if (storageDir == null)
				return;
			if(dstFile != null){
				return;
			}

			String iuid = rq.getString(Tag.AffectedSOPInstanceUID);
			String cuid = rq.getString(Tag.AffectedSOPClassUID);
			String tsuid = pc.getTransferSyntax();
			File file = new File(storageDir, fn);
			try {
				storeTo(as, as.createFileMetaInformation(iuid, cuid, tsuid), data, file);
				dstFile = file;
			} catch (Exception e) {
				throw new DicomServiceException(Status.ProcessingFailure, e);
			}

		}

	};

	private void storeTo(Association as, Attributes fmi, PDVInputStream data, File file) throws IOException {
		file.getParentFile().mkdirs();
		DicomOutputStream out = new DicomOutputStream(file);
		try {
			out.writeFileMetaInformation(fmi);
			data.copyTo(out);
		} finally {
			SafeClose.close(out);
		}
	}

	protected DicomGetImage(String title) {
		super(title);
		DicomServiceRegistry serviceRegistry = new DicomServiceRegistry();
		serviceRegistry.addDicomService(storageSCP);
		device.setDimseRQHandler(serviceRegistry);
	}

	private void download() throws IOException, InterruptedException {

		DimseRSPHandler rspHandler = new DimseRSPHandler(as.nextMessageID()) {

			@Override
			public void onDimseRSP(Association as, Attributes cmd, Attributes data) {
				System.out.println(data);
				super.onDimseRSP(as, cmd, data);
			}
		};
		as.cget(UID.StudyRootQueryRetrieveInformationModelGET, Priority.NORMAL, keys, null, rspHandler);
	}

	public static String toUID(String uid) {
		uid = uid.trim();
		return (uid.equals("*") || Character.isDigit(uid.charAt(0))) ? uid : UID.forName(uid);
	}

	private void configureStorageSOPClass(String cuid, String tsuid) {
		if (!rq.containsPresentationContextFor(cuid))
			rq.addRoleSelection(new RoleSelection(cuid, false, true));
		rq.addPresentationContext(new PresentationContext(2 * rq.getNumberOfPresentationContexts() + 1, cuid, tsuid));

	}

	public File download(String title, String hostname, int port, String imageId, File path, String fn)
			throws IOException, InterruptedException, IncompatibleConnectionException, GeneralSecurityException {
		this.setRemote(title, hostname, port);
		this.conn.setPort(6000);
		this.rq.addPresentationContext(
				new PresentationContext(1, UID.StudyRootQueryRetrieveInformationModelGET, IVR_LE_FIRST));
		configureStorageSOPClass(UID.GrayscaleSoftcopyPresentationStateStorage, UID.ImplicitVRLittleEndian);
		configureStorageSOPClass(UID.GrayscaleSoftcopyPresentationStateStorage, UID.ExplicitVRLittleEndian);
		configureStorageSOPClass(UID.SecondaryCaptureImageStorage, UID.ImplicitVRLittleEndian);
		configureStorageSOPClass(UID.SecondaryCaptureImageStorage, UID.ExplicitVRLittleEndian);
		configureStorageSOPClass(UID.EncapsulatedPDFStorage, UID.ImplicitVRLittleEndian);
		configureStorageSOPClass(UID.EncapsulatedPDFStorage, UID.ExplicitVRLittleEndian);
		configureStorageSOPClass(UID.UltrasoundMultiFrameImageStorage, UID.ImplicitVRLittleEndian);
		configureStorageSOPClass(UID.UltrasoundMultiFrameImageStorage, UID.ExplicitVRLittleEndian);
		configureStorageSOPClass(UID.DigitalXRayImageStorageForPresentation, UID.ImplicitVRLittleEndian);
		configureStorageSOPClass(UID.DigitalXRayImageStorageForPresentation, UID.ExplicitVRLittleEndian);
		configureStorageSOPClass(UID.BasicTextSRStorage, UID.ImplicitVRLittleEndian);
		configureStorageSOPClass(UID.BasicTextSRStorage, UID.ExplicitVRLittleEndian);
		configureStorageSOPClass(UID.CTImageStorage, UID.ImplicitVRLittleEndian);
		configureStorageSOPClass(UID.CTImageStorage, UID.ExplicitVRLittleEndian);
		configureStorageSOPClass(UID.ComputedRadiographyImageStorage, UID.ImplicitVRLittleEndian);
		configureStorageSOPClass(UID.ComputedRadiographyImageStorage, UID.ExplicitVRLittleEndian);
		configureStorageSOPClass(UID.EnhancedSRStorage, UID.ImplicitVRLittleEndian);
		configureStorageSOPClass(UID.EnhancedSRStorage, UID.ExplicitVRLittleEndian);
		configureStorageSOPClass(UID.XRayRadiofluoroscopicImageStorage, UID.ImplicitVRLittleEndian);
		configureStorageSOPClass(UID.XRayRadiofluoroscopicImageStorage, UID.ExplicitVRLittleEndian);
		configureStorageSOPClass(UID.DigitalXRayImageStorageForProcessing, UID.ImplicitVRLittleEndian);
		configureStorageSOPClass(UID.DigitalXRayImageStorageForProcessing, UID.ExplicitVRLittleEndian);
		configureStorageSOPClass(UID.VLPhotographicImageStorage, UID.ImplicitVRLittleEndian);
		configureStorageSOPClass(UID.VLPhotographicImageStorage, UID.ExplicitVRLittleEndian);
		configureStorageSOPClass(UID.VLPhotographicImageStorage, UID.JPEGBaseline1);
		configureStorageSOPClass(UID.KeyObjectSelectionDocumentStorage, UID.ImplicitVRLittleEndian);
		configureStorageSOPClass(UID.KeyObjectSelectionDocumentStorage, UID.ExplicitVRLittleEndian);
		configureStorageSOPClass(UID.UltrasoundImageStorage, UID.ImplicitVRLittleEndian);
		configureStorageSOPClass(UID.KeyObjectSelectionDocumentStorage, UID.ExplicitVRLittleEndian);
		configureStorageSOPClass(UID.VideoPhotographicImageStorage, UID.JPEGBaseline1);
		configureStorageSOPClass(UID.VideoPhotographicImageStorage, UID.MPEG2);
		configureStorageSOPClass(UID.PositronEmissionTomographyImageStorage, UID.ImplicitVRLittleEndian);
		configureStorageSOPClass(UID.PositronEmissionTomographyImageStorage, UID.ExplicitVRLittleEndian);
		configureStorageSOPClass(UID.XRayAngiographicImageStorage, UID.ImplicitVRLittleEndian);
		configureStorageSOPClass(UID.XRayAngiographicImageStorage, UID.ExplicitVRLittleEndian);
		configureStorageSOPClass(UID.DigitalMammographyXRayImageStorageForProcessing, UID.ImplicitVRLittleEndian);
		configureStorageSOPClass(UID.DigitalMammographyXRayImageStorageForProcessing, UID.ExplicitVRLittleEndian);
		configureStorageSOPClass(UID.XRayRadiationDoseSRStorage, UID.ImplicitVRLittleEndian);
		configureStorageSOPClass(UID.XRayRadiationDoseSRStorage, UID.ExplicitVRLittleEndian);
		configureStorageSOPClass(UID.NuclearMedicineImageStorage, UID.ImplicitVRLittleEndian);
		configureStorageSOPClass(UID.NuclearMedicineImageStorage, UID.ExplicitVRLittleEndian);
		configureStorageSOPClass(UID.VLWholeSlideMicroscopyImageStorage, UID.JPEGBaseline1);
		configureStorageSOPClass(UID.VLWholeSlideMicroscopyImageStorage, UID.JPEG2000LosslessOnly);
		configureStorageSOPClass(UID.VLWholeSlideMicroscopyImageStorage, UID.JPEG2000);
		configureStorageSOPClass(UID.MRImageStorage, UID.ImplicitVRLittleEndian);
		configureStorageSOPClass(UID.MRImageStorage, UID.ExplicitVRLittleEndian);
		configureStorageSOPClass(UID.DigitalMammographyXRayImageStorageForPresentation, UID.ImplicitVRLittleEndian);
		configureStorageSOPClass(UID.DigitalMammographyXRayImageStorageForPresentation, UID.ExplicitVRLittleEndian);
		this.keys.setString(Tag.QueryRetrieveLevel, VR.CS, "IMAGE");

		this.addAttribute(Tag.SOPInstanceUID, imageId);

		this.storageDir = path;
		this.fn = fn;

		ExecutorService executorService = Executors.newSingleThreadExecutor();
		ScheduledExecutorService scheduledExecutorService = Executors.newSingleThreadScheduledExecutor();
		this.device.setExecutor(executorService);
		this.device.setScheduledExecutor(scheduledExecutorService);

		try {
			as = ae.connect(conn, remote, rq);
			download();
		} finally {
			if (as != null && as.isReadyForDataTransfer()) {
				as.waitForOutstandingRSP();
				as.release();
			}
			executorService.shutdown();
			scheduledExecutorService.shutdown();
		}
		return this.dstFile;
	}

	public static void main(String args[])
			throws IOException, InterruptedException, IncompatibleConnectionException, GeneralSecurityException {
		DicomGetImage q = new DicomGetImage("AIADS");
		q.download("STPACS", "133.0.14.150", 104, "1.2.840.113663.1500.1.446929507.3.1.20190308.101952.92",
				new File("/ekexiu/storage_asd/download"), "1.2.840.113663.1500.1.446929507.3.1.20190308.101952.92.dcm");

	}

}
