package com.ekexiu.project.storage.dicom;

import java.io.IOException;
import java.security.GeneralSecurityException;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;

import org.dcm4che3.data.Attributes;
import org.dcm4che3.data.ElementDictionary;
import org.dcm4che3.data.Tag;
import org.dcm4che3.data.UID;
import org.dcm4che3.data.VR;
import org.dcm4che3.net.Association;
import org.dcm4che3.net.DimseRSPHandler;
import org.dcm4che3.net.IncompatibleConnectionException;
import org.dcm4che3.net.Priority;
import org.dcm4che3.net.Status;
import org.dcm4che3.net.pdu.PresentationContext;
import org.jfw.util.json.JsonService;

public class DicomQueryStudy extends DicomBase {

	private List<PatientStudy> result = new LinkedList<PatientStudy>();
	private PatientStudy study;
	private StringBuilder sb = new StringBuilder();


	private Attributes.Visitor visitor = new Attributes.Visitor() {

		public boolean visit(Attributes attrs, int tag, VR vr, Object value) throws Exception {
			if (value != null) {
				sb.setLength(0);
				if(vr.prompt(value, attrs.bigEndian(), attrs.getSpecificCharacterSet(vr),
		               100000, sb)){
					if(sb.length()<1){
						return true;
					}
					if(tag == Tag.PatientName){
						study.setPatientName(sb.toString());
					}else if(tag == Tag.PatientID){
						study.setPatientId(sb.toString());
					}else if(tag == Tag.PatientSex){
						study.setSex(sb.toString());
					}else if(tag == Tag.PatientBirthDate){
						study.setBirth(sb.toString());
					}else if(tag == Tag.StudyInstanceUID){
						study.setStudyUid(sb.toString());
					}else if(tag == Tag.StudyID){
						study.setStudyId(sb.toString());
					}else if(tag == Tag.StudyDate){
						study.setStudyDate(sb.toString());
					}else if(tag == Tag.StudyTime){
						study.setStudyTime(sb.toString());
					}else if(tag == Tag.StudyDescription){
						study.setDescription(sb.toString());
					}else if(tag == Tag.NumberOfStudyRelatedSeries){
						study.setNumOfStudyRelatedSeries(sb.toString());
					}else if(tag == Tag.NumberOfStudyRelatedInstances){
						study.setNumOfstudyRelatedInstances(sb.toString());
					}
				}
			}
			return true;
		}
	};

	protected DicomQueryStudy(String title) {
		super(title);
	}

	private void query() throws IOException, InterruptedException {
		DimseRSPHandler rspHandler = new DimseRSPHandler(as.nextMessageID()) {
			@Override
			public void onDimseRSP(Association as, Attributes cmd, Attributes data) {
				super.onDimseRSP(as, cmd, data);
				int status = cmd.getInt(Tag.Status, -1);
				if (Status.isPending(status)) {
					study = new PatientStudy();
					result.add(study);
					try {
						data.accept(visitor, false);
					} catch (Exception e) {
					}
				}
			}
		};
		as.cfind(UID.StudyRootQueryRetrieveInformationModelFIND, Priority.NORMAL, keys, null, rspHandler);
	}

	public List<PatientStudy> query(String title, String hostname, int port, String patientName, String patientId)
			throws IOException, InterruptedException, IncompatibleConnectionException, GeneralSecurityException {
		this.setRemote(title, hostname, port);
		this.rq.addPresentationContext(
				new PresentationContext(1, UID.StudyRootQueryRetrieveInformationModelFIND, IVR_LE_FIRST));
		this.keys.setString(Tag.QueryRetrieveLevel, VR.CS, "STUDY");
		this.addAttribute(Tag.PatientID, patientId);
		this.addAttribute(Tag.PatientName, patientName);
		this.addAttribute(Tag.IssuerOfPatientID, null);
		this.addAttribute(Tag.PatientSex, null);
		this.addAttribute(Tag.PatientBirthDate, null);
		this.addAttribute(Tag.StudyInstanceUID, null);
		this.addAttribute(Tag.StudyInstanceUID, null);
		this.addAttribute(Tag.SeriesInstanceUID, null);
		this.addAttribute(Tag.StudyID, null);
		this.addAttribute(Tag.AccessionNumber, null);
		this.addAttribute(Tag.StudyDate, null);
		this.addAttribute(Tag.StudyTime, null);
		this.addAttribute(Tag.StudyDescription, null);
		this.addAttribute(Tag.NumberOfStudyRelatedSeries, null);
		this.addAttribute(Tag.NumberOfStudyRelatedInstances, null);
		ExecutorService executorService = Executors.newSingleThreadExecutor();
		ScheduledExecutorService scheduledExecutorService = Executors.newSingleThreadScheduledExecutor();
		this.device.setExecutor(executorService);
		this.device.setScheduledExecutor(scheduledExecutorService);

		try {
			as = ae.connect(conn, remote, rq);
			query();
		} finally {
			if (as != null && as.isReadyForDataTransfer()) {
				as.waitForOutstandingRSP();
				as.release();
			}
			executorService.shutdown();
			scheduledExecutorService.shutdown();
		}
		return this.result;
	}

	public static void main(String args[])
			throws IOException, InterruptedException, IncompatibleConnectionException, GeneralSecurityException {
		DicomQueryStudy q = new DicomQueryStudy("AIADS");
		q.query("STPACS", "133.0.14.150", 104,null, /*"759417ASD"*/"PID000115788");
		
		for(PatientStudy s:q.result){
			System.out.println(JsonService.toJson(s));
		}

	}
}
